package mixer;

import java.util.ArrayList;
import java.util.List;

class Graph
{
	private int n;
	private Node[] nodes;
	protected Graph(int[] permutation){
		n = permutation.length; // n = 2^k
        createNodes();
        createEdges(permutation);
        setSwitches();
	}
	
	protected boolean getSwitchValue(int index,boolean up)
    {
		// index must be less then n/2
        return up ? nodes[index].on : nodes[index + n / 2].on;
    }
	
	private void createNodes()
    {
        nodes = new Node[n];
        for (int i = 0; i < n / 2; i++)
        {
            nodes[i] = new Node(i, i + n / 2, true);
            nodes[i + n / 2] = new Node(i, i + n / 2, false);
        }
    }
	
	private void createEdges(int[] permutation)
    {
        int pi1, pi2;
        for (int i = 0; i < n / 2; i++)
        {
            pi1 = (permutation[i] < n / 2) ? permutation[i] + (n / 2) : permutation[i];
            pi2 = (permutation[i + n / 2] < n / 2) ? permutation[i + n / 2] + (n / 2) : permutation[i + n / 2];

            nodes[i].edges.add(new Edge(nodes[pi1], (permutation[i] >= n / 2)));
            nodes[pi1].edges.add(new Edge(nodes[i], (permutation[i] >= n / 2)));

            nodes[i].edges.add(new Edge(nodes[pi2], (permutation[i + n / 2] < n / 2)));
            nodes[pi2].edges.add(new Edge(nodes[i], (permutation[i + n / 2] < n / 2)));
        }
    }
	
	private void setSwitches()
    {
        for (int i = 0; i < n / 2; i++)
        {
            Node node = nodes[i];
            if (node.set)
                continue;
            boolean v = false;
            while (true)
            {
                node.set = true;
                node.on = v;
                
                if (node.edges.get(0).nighbor.set && node.edges.get(1).nighbor.set)
                    break;
                v ^= (!node.edges.get(0).nighbor.set) ? node.edges.get(0).broken : node.edges.get(1).broken;
                node = (!node.edges.get(0).nighbor.set) ? node.edges.get(0).nighbor : node.edges.get(1).nighbor;
            }
        }
    }
	
	private class Node
	{
		public boolean up;
		public List<Edge> edges;
		public int i, j;
		public boolean on;
		public boolean set;
		public Node(int i, int j,boolean up)
		{
			this.i = i;
			this.j = j;
			this.up = up;
			edges = new ArrayList<Edge>();
			set = false;
		} 
	}

	private class Edge
	{
		public Node nighbor;
		public boolean broken;
		public Edge(Node nighbor, boolean broken)
		{
			this.nighbor = nighbor;
			this.broken = broken;
		}
	}

	
}

